/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.base.model.element.IModelElement;
import org.fortiss.tooling.kernel.model.IProjectRootElement;
import org.fortiss.tooling.kernel.service.base.EObjectAwareServiceBase;
import org.fortiss.tooling.kernel.utils.EcoreUtils;
import org.fortiss.tooling.kernel.utils.KernelModelElementUtils;

/**
 * 
 * @author eder
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class AnnotationValueService extends
		EObjectAwareServiceBase<IAnnotationValueProvider<IAnnotatedSpecification>> implements
		IAnnotationValueService {

	/** The compositor extension point ID. */
	private static final String EXTENSION_POINT_NAME =
			"org.fortiss.tooling.base.ui.annotationViewPart";

	/** The compositor configuration element name. */
	private static final String CONFIGURATION_ELEMENT_NAME = "annotationViewPart";

	/** The handler class attribute name. */
	private static final String HANDLER_CLASS_ATTRIBUTE_NAME = "binding";

	/**
	 * 
	 */
	public AnnotationValueService() {
		super();
	}

	/** {@inheritDoc} */
	@Override
	public List<AnnotationEntry<? extends IAnnotatedSpecification>> getValues(
			final IModelElement element) {
		if(element == null) {
			return Collections.emptyList();
		}
		final List<AnnotationEntry<? extends IAnnotatedSpecification>> result =
				new ArrayList<AnnotationEntry<? extends IAnnotatedSpecification>>();// Collections.emptyMap();
		final List<IAnnotationValueProvider<IAnnotatedSpecification>> registeredHandlers =
				getRegisteredHandlers(element.getClass());
		if(registeredHandlers != null && !registeredHandlers.isEmpty()) {

			// getAllChildren(element, clazz, exceptions)

			// TreeIterator<EObject> eAllContents = element.eAllContents();
			// EList<EObject> eContents = element.eContents();
			// List<? extends IModelElement> allChildren =
			// getAllChildren(element, element.getClass(), null);
			// final IAnnotationValueProvider<IAnnotatedSpecification> annotationProvider =
			// registeredHandlers.get(0);
			KernelModelElementUtils.runAsCommand(element, new Runnable() {

				@Override
				public void run() {
					EObject root = element;
					while(!(root instanceof IProjectRootElement)) {
						root = root.eContainer();
					}
					final List<? extends IModelElement> childrenWithType =
							EcoreUtils.getChildrenWithType(root, element.getClass());
					for(IAnnotationValueProvider<IAnnotatedSpecification> annotationProvider : registeredHandlers) {

						List<? extends IModelElement> allChildren =
								getAllChildren(root, element.getClass(),
										annotationProvider.filterOut());

						for(final IModelElement e : allChildren) {
							EList<IAnnotatedSpecification> annotatedSpecifications =
									EcoreUtils.pickInstanceOf(IAnnotatedSpecification.class,
											e.getSpecifications());
							IAnnotatedSpecification annotatedSpec = null;
							for(IAnnotatedSpecification spec : annotatedSpecifications) {
								if(spec.getClass().isInstance(annotationProvider)) {
									annotatedSpec = spec;
								}
							}

							if(annotatedSpec == null) {
								annotatedSpec = annotationProvider.addNewSpecToModelElement(e);
							}

							AnnotationEntry<? extends IAnnotatedSpecification> entry =
									new AnnotationEntry<IAnnotatedSpecification>(e, annotatedSpec,
											annotationProvider);
							result.add(entry);
							// if(annotatedSpec.getClass().isInstance(annotationProvider)) {
							// result.put(e, annotationProvider.getValues(annotatedSpec));
							// }

						}
					}
				}
			});
			return result;
		}
		return result;
	}

	private <S, T> List<T> getAllChildren(EObject element, Class<? extends IModelElement> class1,
			List<Class<? extends EObject>> filterOut) {

		List<T> results = new ArrayList<T>();
		getAllChildren2(element, class1, filterOut, results);
		// EList<EObject> eContents = element.eContents();
		// for(EObject o : eContents) {
		// if(o.getClass().isInstance(clazz))
		// getAllChildren((IModelElement)o, clazz, exceptions).add((T)o);
		// else {
		// if(o instanceof IModelElement)
		// getAllChildren((IModelElement)o, clazz, exceptions);
		// }
		// }

		return results;

	}

	private <T, S> void getAllChildren2(EObject element, Class<? extends IModelElement> class1,
			List<Class<? extends EObject>> filterOut, final List<T> results) {

		EList<EObject> eContents = element.eContents();
		for(EObject o : eContents) {
			if(class1.isInstance(o)) {
				results.add((T)o);
				getAllChildren2(o, class1, filterOut, results);
			} else {
				boolean filter = false;
				for(Class<?> c : filterOut) {
					if(c.isInstance(o))
						filter = true;
				}
				if(!filter)
					getAllChildren2(o, class1, filterOut, results);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	protected String getExtensionPointName() {
		return EXTENSION_POINT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getConfigurationElementName() {
		return CONFIGURATION_ELEMENT_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected String getHandlerClassAttribute() {
		return HANDLER_CLASS_ATTRIBUTE_NAME;
	}

	/** {@inheritDoc} */
	@Override
	protected boolean requiresUniqueHandler() {
		return false;
	}

}
