/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation;

import java.util.HashMap;
import java.util.Map.Entry;

import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Table;
import org.eclipse.ui.IWorkbenchPart;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;

/**
 * 
 * @author eder
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class DummyAnnotationView extends AnnotationViewPartBase2 {

	/** mapping of {@link IAnnotatedSpecification} -> TableViewerColumn */
	private HashMap<Class<? extends IAnnotatedSpecification>, TableViewerColumn> columns =
			new HashMap<Class<? extends IAnnotatedSpecification>, TableViewerColumn>();

	/** The table viewer */
	private TableViewer tableViewer;

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		super.createPartControl(parent);
		Composite c = new Composite(parent, SWT.NULL);
		c.setLayout(new FillLayout(SWT.HORIZONTAL));

		tableViewer = new TableViewer(c, SWT.BORDER | SWT.FULL_SELECTION);
		Table table = tableViewer.getTable();
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		final TableViewerColumn firstColumn = new TableViewerColumn(tableViewer, SWT.NONE);
		firstColumn.getColumn().setText("IModelElement");
		firstColumn.getColumn().setWidth(100);

		tableViewer.setContentProvider(new ArrayContentProvider());
		firstColumn.setLabelProvider(new ColumnLabelProvider() {
			/** {@inheritDoc} */
			@Override
			public String getText(Object element) {
				AnnotationEntry<?> entry = (AnnotationEntry<?>)element;
				String modelElementName = entry.getModelElement().getClass().getSimpleName();
				modelElementName = modelElementName.replace("Impl", "");
				firstColumn.getColumn().setText(modelElementName);
				return ((INamedCommentedElement)entry.getModelElement()).getName();// toString();

			}
		});

	}

	/** {@inheritDoc} */
	@Override
	public void selectionChanged(IWorkbenchPart part, ISelection selection) {
		super.selectionChanged(part, selection);

		for(Entry<Class<? extends IAnnotatedSpecification>, TableViewerColumn> e : columns
				.entrySet()) {
			e.getValue().getColumn().dispose();
		}
		columns.clear();

		for(AnnotationEntry<? extends IAnnotatedSpecification> entry : values) {
			if(!isExistingColumn(entry.getSpecification())) {
				TableViewerColumn column = new TableViewerColumn(tableViewer, SWT.NONE);
				column.getColumn().setText(entry.getAnnotationName());
				column.getColumn().setWidth(100);
				columns.put(entry.getSpecification().getClass(), column);
				column.setLabelProvider(new ColumnLabelProvider2());
			}

		}

		if(!tableViewer.getTable().isDisposed())
			tableViewer.setInput(values);

	}

	/** Checks if column for given specification already exists */
	private boolean isExistingColumn(IAnnotatedSpecification specification) {
		if(columns.containsKey(specification.getClass()))
			return true;
		return false;
	}

	private class ColumnLabelProvider2 extends ColumnLabelProvider {

		/** {@inheritDoc} */
		@Override
		public String getText(Object element) {
			AnnotationEntry<?> entry = (AnnotationEntry<?>)element;
			return entry.getSpecificationValue();
		}
	}

	// FIXME
	// public final class ExampleEditingSupport extends EditingSupport {
	//
	// private ComboBoxViewerCellEditor cellEditor = null;
	//
	// private ExampleEditingSupport(ColumnViewer viewer) {
	// super(viewer);
	// cellEditor =
	// new ComboBoxViewerCellEditor((Composite)getViewer().getControl(), SWT.READ_ONLY);
	// cellEditor.setLabelProvider(new LabelProvider());
	// cellEditor.setContentProvider(new ArrayContentProvider());
	// cellEditor.setInput(Value.values());
	// }
	//
	// @Override
	// protected CellEditor getCellEditor(Object element) {
	// return cellEditor;
	// }
	//
	// @Override
	// protected boolean canEdit(Object element) {
	// return true;
	// }
	//
	// @Override
	// protected Object getValue(Object element) {
	// if(element instanceof ExampleData) {
	// ExampleData data = (ExampleData)element;
	// return data.getData();
	// }
	// return null;
	// }
	//
	// @Override
	// protected void setValue(Object element, Object value) {
	// if(element instanceof ExampleData && value instanceof Value) {
	// ExampleData data = (ExampleData)element;
	// Value newValue = (Value)value;
	// /* only set new value if it differs from old one */
	// if(!data.getData().equals(newValue)) {
	// data.setData(newValue);
	// }
	// }
	// }
	//
	// }
	//
	// public static enum Value {
	// value1, value2, value3;
	// }
	//
	// public class ExampleData {
	//
	// private String label;
	// private Value value;
	//
	// public ExampleData(String label, Value data) {
	// this.label = label;
	// this.value = data;
	// }
	//
	// public String getLabel() {
	// return label;
	// }
	//
	// public void setLabel(String label) {
	// this.label = label;
	// }
	//
	// public Value getData() {
	// return value;
	// }
	//
	// public void setData(Value data) {
	// this.value = data;
	// }
	//
	// }
}
